/*--
     server.c

     Protocol on the wire is as follows;

     4 byte header  - specifies length of data to follow
     data           - the actual data

     Server, when running on Windows, will perform
     ASCII<-->EBCDIC conversion
--*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#ifdef WIN32
  #include "winsock2.h"
#else
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <netdb.h>
#endif

#ifdef WIN32
  /*-- define function mappings --*/
  #define close(x)            closesocket(x)
  #define read(x, y, z)       recv( (x), (y), (z), 0)
  #define write(x, y, z)      send( (x), (y), (z), 0)

  /*-- Winsock setup stuff --*/
  static WSADATA wskData;
  static int winSockInit;

  long convertAsciiToEbcdic(unsigned char *inBuffer, long inLength,
                          unsigned char *outBuffer, long outLength);
  long convertEbcdicToAscii(unsigned char *inBuffer, long inLength,
                          unsigned char *outBuffer, long outLength);
#endif

char *show_address(struct in_addr *myaddr);

int main(int argc, char**argv)
{
    unsigned short  portnumber;
    int     len;
    struct  sockaddr_in addr;
    int     socketlisten,
            liveSocket = 0,
            i,
            theReuseOption;

    /*-- need to set this to a real port --*/
    portnumber = 8888;

    socketlisten = socket(AF_INET, SOCK_STREAM, 0);
    if (socketlisten == -1)
    {
        fprintf(stderr, "TCP ERROR : Could not create socket\n");
        return -1;
    }

    theReuseOption = 1;

    i = setsockopt(socketlisten, SOL_SOCKET, SO_REUSEADDR,
                  (char *)&theReuseOption, sizeof(theReuseOption));

    memset(&addr, 0, sizeof(addr));
    addr.sin_family      = PF_INET;
    addr.sin_port        = htons(portnumber);
    addr.sin_addr.s_addr = INADDR_ANY;
    len = sizeof(addr);
    i  = 0;

    if (bind(socketlisten, (void*)&addr, len) > 0)
    {
        fprintf(stderr, "TCP ERROR : Could not bind to socket\n");
        close(socketlisten);
        return -1;
    }

    if (listen(socketlisten,1) > 0)
    {
        fprintf(stderr, "TCP ERROR : Could not listen on socket\n");
        close(socketlisten);
        return -1;
    }

    /*-- keep going --*/
    while(1)
    {
        liveSocket = accept( socketlisten, (void *)&addr, &len);

        if (liveSocket == -1)
        {
            fprintf(stderr, "TCP ERROR : Could not accept socket\n");
            close(socketlisten);
            return -1;
        }
        else
        {
            long dataLen1, dataLen2, i, l;
            unsigned char   *msgPtr;
            unsigned char   *msgData;
            unsigned char   *replyPtr;
            unsigned char   replyData[] =
                            "some test data from server";

            /*-- connection accepted, get data length--*/
            i = read(liveSocket, (void *)&l, 4);

            dataLen1 = ntohl(l);
            dataLen2 = dataLen1;

            /*-- allocate memory for data --*/
            msgData = calloc(dataLen1 + 1, 1);
            msgPtr  = msgData;

            /*-- get data, loop around until all read --*/
            while(dataLen1)
            {
                i = read(liveSocket, msgPtr, dataLen1);
                dataLen1 -= i;
                msgPtr   += dataLen1;
            }

            /*-- At this point, msgData now
                 contains the data sent from the
                 other end of the socket
            --*/
            #ifdef WIN32
                /*-- convert ebcdic to ascii --*/
                convertEbcdicToAscii(msgData, dataLen2,
                                     msgData, dataLen2);
            #endif
            fprintf(stdout, "Message from client :\n %s\n", msgData);

            /*-- finished with data --*/
            free(msgData);

            /*--
                 Let's send the reply to the
                 other end of the socket
            --*/
            dataLen1 = strlen(replyData);
            i = htonl(dataLen1);

            /*-- send length of data --*/
            write(liveSocket, (void *)&i, 4);

            #ifdef WIN32
                /*-- convert ascii to ebcdic --*/
                convertAsciiToEbcdic(replyData, dataLen1,
                                     replyData, dataLen1);
            #endif
            /*-- send the data --*/
            replyPtr = replyData;

            /*-- loop around until all data is sent --*/
            while(dataLen1)
            {
                i = write(liveSocket, replyPtr, dataLen1);
                dataLen1 -= i;
                replyPtr += i;
            }
        }
    }
    return 0;
}
